searchData={"items":[{"type":"module","title":"ContextKit","doc":"ContextKit provides a modular toolkit for building robust Phoenix/Ecto contexts with standardized CRUD operations.","ref":"ContextKit.html"},{"type":"module","title":"Overview - ContextKit","doc":"ContextKit aims to reduce boilerplate code in Phoenix applications by providing:\n\n- Standardized CRUD operation generators\n- Dynamic query building with extensive filtering options\n- Built-in pagination support\n- Support for Phoenix 1.8 scopes\n- PubSub integration for real-time updates\n- Flexible and extensible design","ref":"ContextKit.html#module-overview"},{"type":"module","title":"Basic Usage - ContextKit","doc":"ContextKit offers two main modules:\n\n- [`ContextKit.CRUD`](ContextKit.CRUD.html) - Core CRUD operations with no scope support\n- [`ContextKit.CRUD.Scoped`](ContextKit.CRUD.Scoped.html) - CRUD operations with Phoenix 1.8 scope support and PubSub integration\n\nChoose the module that fits your needs:\n\n- Use `ContextKit.CRUD` for simple contexts like in Phoenix 1.7 without scopes or PubSub needs\n- Use `ContextKit.CRUD.Scoped` when working with Phoenix 1.8 scopes and wanting real-time updates","ref":"ContextKit.html#module-basic-usage"},{"type":"module","title":"Getting Started - ContextKit","doc":"","ref":"ContextKit.html#module-getting-started"},{"type":"module","title":"1. First, define your schema modules: - ContextKit","doc":"```elixir\ndefmodule MyApp.Accounts.User do\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  schema \"users\" do\n    field :email, :string\n    field :name, :string\n    field :status, :string\n\n    has_many :comments, MyApp.Blog.Comment\n\n    timestamps()\n  end\n\n  def changeset(user, attrs) do\n    user\n    |> cast(attrs, [:email, :name, :status])\n    |> validate_required([:email, :name])\n    |> unique_constraint(:email)\n  end\nend\n\ndefmodule MyApp.Blog.Comment do\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  schema \"comments\" do\n    field :body, :string\n    field :status, :string\n\n    belongs_to :user, MyApp.Accounts.User\n\n    timestamps()\n  end\n\n  # Standard changeset for unscoped operations\n  def changeset(comment, attrs) do\n    comment\n    |> cast(attrs, [:body, :status, :user_id])\n    |> validate_required([:body, :user_id])\n  end\n\n  # Scoped changeset for Phoenix 1.8 scope operations\n  def changeset(comment, attrs, scope) do\n    comment\n    |> cast(attrs, [:body, :status])\n    |> validate_required([:body])\n    |> put_change(:user_id, scope.user.id)\n  end\nend\n```","ref":"ContextKit.html#module-1-first-define-your-schema-modules"},{"type":"module","title":"2. Create a queries module for custom query logic: - ContextKit","doc":"```elixir\ndefmodule MyApp.Blog.CommentQueries do\n  import Ecto.Query\n\n  def apply_query_option({:with_recent_activity, true}, query) do\n    query\n    |> where([c], c.inserted_at > ago(1, \"day\"))\n  end\n\n  def apply_query_option({:with_user_name, name}, query) do\n    query\n    |> join(:inner, [c], u in assoc(c, :user))\n    |> where([_, u], u.name == ^name))\n  end\nend\n```","ref":"ContextKit.html#module-2-create-a-queries-module-for-custom-query-logic"},{"type":"module","title":"3. Use `ContextKit.CRUD` for basic contexts: - ContextKit","doc":"```elixir\n# Basic usage without scopes\ndefmodule MyApp.Blog do\n  use ContextKit.CRUD,\n    repo: MyApp.Repo,\n    schema: MyApp.Blog.Comment,\n    queries: MyApp.Blog.CommentQueries\nend\n```","ref":"ContextKit.html#module-3-use-contextkit-crud-for-basic-contexts"},{"type":"module","title":"4. Or use `ContextKit.CRUD.Scoped` for Phoenix 1.8 Scopes and PubSub: - ContextKit","doc":"```elixir\ndefmodule MyApp.Blog do\n  use ContextKit.CRUD.Scoped,\n    repo: MyApp.Repo,\n    schema: MyApp.Blog.Comment,\n    queries: MyApp.Blog.CommentQueries,\n    pubsub: MyApp.PubSub,\n    scope: Application.compile_env(:my_app, :scopes)[:user]\nend\n```\n\nYou can define custom query functions directly in your context module by setting `queries: __MODULE__`,\neliminating the need for a separate queries module. This is convenient for simpler contexts\nwhere you don't need to share query logic.","ref":"ContextKit.html#module-4-or-use-contextkit-crud-scoped-for-phoenix-1-8-scopes-and-pubsub"},{"type":"module","title":"Features - ContextKit","doc":"","ref":"ContextKit.html#module-features"},{"type":"module","title":"Standard CRUD Operations - ContextKit","doc":"ContextKit automatically generates common CRUD functions:\n\n```elixir\n# Get query for use with Repo functions\nBlog.query_comments()\nBlog.query_comments(status: \"published\")\n\n# New record\nBlog.new_comment(%{status: \"published\"})\n\n# List records with filtering and pagination\nBlog.list_comments(status: \"published\", paginate: [page: 1, per_page: 20])\n\n# Get single record\nBlog.get_comment(123)\nBlog.get_comment!(123)  # Raises if not found\n\n# Get one record by criteria\nBlog.one_comment(user_id: 1)\nBlog.one_comment!(user_id: 1)  # Raises if not found\n\n# Save records (insert or update based on record state)\nBlog.save_comment(comment, %{body: \"New or updated content\"})\nBlog.save_comment!(comment, %{body: \"New or updated content\"})  # Raises on error\n\n# Create records\nBlog.create_comment(%{body: \"Great post!\", user_id: 1})\nBlog.create_comment!(%{body: \"Great post!\", user_id: 1})  # Raises on error\n\n# Update records\nBlog.update_comment(comment, %{body: \"Updated comment\"})\nBlog.update_comment!(comment, %{body: \"Updated comment\"})  # Raises on error\n\n# Get changeset\nBlog.change_comment(comment)\nBlog.change_comment(comment, %{body: \"Changed comment\"})\n\n# Delete records\nBlog.delete_comment(comment)\nBlog.delete_comment(user_id: 1, body: \"Specific comment\")\n```","ref":"ContextKit.html#module-standard-crud-operations"},{"type":"module","title":"Scoped Operations (Phoenix 1.8) - ContextKit","doc":"When using `ContextKit.CRUD.Scoped`, all CRUD operations can take a scope parameter:\n\n```elixir\n# Get query scoped to current user\nBlog.query_comments(socket.assigns.current_scope)\nBlog.query_comments(socket.assigns.current_scope, status: \"published\")\n\n# List comments for the current user\nBlog.list_comments(socket.assigns.current_scope)\n\n# Get comment by ID only if it belongs to the current user\nBlog.get_comment(socket.assigns.current_scope, 123)\n\n# Save comment (insert or update) with scope\nBlog.save_comment(socket.assigns.current_scope, comment, %{body: \"New or updated content\"})\n\n# Create comment for the current user\nBlog.create_comment(socket.assigns.current_scope, %{body: \"Great post!\"})\n\n# Update comment only if it belongs to the current user\nBlog.update_comment(socket.assigns.current_scope, comment, %{body: \"Updated comment\"})\n\n# Delete comment only if it belongs to the current user\nBlog.delete_comment(socket.assigns.current_scope, comment)\n```","ref":"ContextKit.html#module-scoped-operations-phoenix-1-8"},{"type":"module","title":"Advanced Filtering - ContextKit","doc":"Supports a wide range of filter operations:\n\n```elixir\n# Basic equality\nBlog.list_comments(status: \"published\")\n\n# Complex filters\nBlog.list_comments(filters: [\n  %{field: :body, op: :ilike, value: \"%awesome%\"},\n  %{field: :status, op: :in, value: [\"published\", \"pending\"]},\n  %{field: :inserted_at, op: :gt, value: ~N[2023-01-01 00:00:00]}\n])\n```\n\nAll fields from the schema can be filtered on automatically.\nAny option not recognized as a field filter or standard query option is treated as a custom query option and passed to\nthe queries module's `apply_query_option/2` function.","ref":"ContextKit.html#module-advanced-filtering"},{"type":"module","title":"Query Operations - ContextKit","doc":"ContextKit provides query functions that return Ecto queries without executing them,\nuseful for aggregation, composition, or further customization:\n\n```elixir\n# Get a base query for all comments\nquery = Blog.query_comments()\n\n# Apply filters to a query\nquery = Blog.query_comments(status: \"published\")\n\n# Get a scoped query (with ContextKit.CRUD.Scoped)\nquery = Blog.query_comments(socket.assigns.current_scope)\n\n# Use with Repo functions\nMyApp.Repo.aggregate(query, :count)\nMyApp.Repo.all(query)\n\n# Compose with other queries\nquery\n|> join(:left, [c], u in assoc(c, :user))\n|> group_by([c, u], u.id)\n|> select([c, u], {u.name, count(c.id)})\n|> MyApp.Repo.all()\n```\n\nQuery functions return an `Ecto.Query` which is perfect for:\n- Computing aggregations (count, sum, etc.)\n- Creating complex reports with multiple joins\n- Building sub-queries\n- Performing custom operations that ContextKit's standard functions don't cover","ref":"ContextKit.html#module-query-operations"},{"type":"module","title":"Pagination - ContextKit","doc":"Built-in pagination support:\n\n```elixir\n{comments, pagination} = Blog.list_comments(\n  status: \"published\",\n  paginate: [page: 2, per_page: 20]\n)\n\n# pagination struct includes:\n# - total_count\n# - total_pages\n# - current_page\n# - per_page\n# - has_next_page?\n# - has_previous_page?\n# - next_page\n# - previous_page\n```","ref":"ContextKit.html#module-pagination"},{"type":"module","title":"PubSub Integration - ContextKit","doc":"When using `ContextKit.CRUD.Scoped` with `pubsub` and `scope` options, automatic broadcasting occurs for create, update, and delete operations:\n\n```elixir\n# Subscribe to scoped updates\nBlog.subscribe_comments(socket.assigns.current_scope)\n\n# Now the current process will receive messages like:\n# {:created, %Comment{}}\n# {:updated, %Comment{}}\n# {:deleted, %Comment{}}\n\n# Broadcast custom messages\nBlog.broadcast_comment(socket.assigns.current_scope, {:custom_event, comment})\n```","ref":"ContextKit.html#module-pubsub-integration"},{"type":"module","title":"Custom Query Options - ContextKit","doc":"Extend with custom query logic:\n\n```elixir\n# In your queries module\ndef apply_query_option({:with_recent_activity, true}, query) do\n  query\n  |> where([c], c.inserted_at > ago(1, \"day\"))\nend\n\n# Usage\nBlog.list_comments(with_recent_activity: true)\n```","ref":"ContextKit.html#module-custom-query-options"},{"type":"module","title":"Configuration Options - ContextKit","doc":"When using `ContextKit.CRUD` or `ContextKit.CRUD.Scoped`, you can configure:\n\n- `repo`: Your Ecto repository module\n- `schema`: The Ecto schema module\n- `queries`: Module containing custom query functions\n- `except`: List of operations to exclude (`:new`, `:list`, `:get`, `:one`, `:delete`, `:create`, `:update`, `:change`, `:subscribe`, `:broadcast`)\n- `plural_resource_name`: Custom plural name for list functions\n\nAdditional options for `ContextKit.CRUD.Scoped`:\n\n- `pubsub`: Phoenix.PubSub module for real-time features\n- `scope`: Configuration for Phoenix 1.8 scopes","ref":"ContextKit.html#module-configuration-options"},{"type":"module","title":"Best Practices - ContextKit","doc":"1. Choose the right module: `ContextKit.CRUD` for simple contexts, `ContextKit.CRUD.Scoped` for contexts with scopes\n2. Create separate query modules for complex filtering logic\n3. Override generated functions when you need custom behavior\n4. Use pagination for large datasets\n5. When using scopes, implement schema-level changeset functions that accept a scope parameter\n6. Use PubSub for real-time updates in LiveView applications","ref":"ContextKit.html#module-best-practices"},{"type":"module","title":"ContextKit.CRUD","doc":"The `ContextKit.CRUD` module provides a convenient way to generate standard CRUD (Create, Read, Update, Delete)\noperations for your Ecto schemas. It reduces boilerplate code by automatically generating commonly used database\ninteraction functions.","ref":"ContextKit.CRUD.html"},{"type":"module","title":"Setup - ContextKit.CRUD","doc":"Add the following to your context module:\n\n```elixir\ndefmodule MyApp.Blog do\n  use ContextKit.CRUD,\n    repo: MyApp.Repo,\n    schema: MyApp.Blog.Comment,\n    queries: MyApp.Blog.CommentQueries,\n    except: [:delete],                    # Optional: exclude specific operations\n    plural_resource_name: \"comments\"      # Optional: customize plural name\nend\n```","ref":"ContextKit.CRUD.html#module-setup"},{"type":"module","title":"Required Options - ContextKit.CRUD","doc":"* `:repo` - The Ecto repository module to use for database operations\n  * `:schema` - The Ecto schema module that defines your resource\n  * `:queries` - Module containing query-building functions for advanced filtering","ref":"ContextKit.CRUD.html#module-required-options"},{"type":"module","title":"Optional Options - ContextKit.CRUD","doc":"* `:except` - List of operation types to exclude (`:list`, `:get`, `:one`, `:delete`, `:create`, `:update`, `:change`)\n  * `:plural_resource_name` - Custom plural name for list functions (defaults to singular + \"s\")","ref":"ContextKit.CRUD.html#module-optional-options"},{"type":"module","title":"Generated Functions - ContextKit.CRUD","doc":"For a schema named `Comment`, the following functions are generated:","ref":"ContextKit.CRUD.html#module-generated-functions"},{"type":"module","title":"Query Operations - ContextKit.CRUD","doc":"* `query_comments/0` - Returns a base query for all comments\n  * `query_comments/1` - Returns a filtered query based on options (without executing)","ref":"ContextKit.CRUD.html#module-query-operations"},{"type":"module","title":"New Operations - ContextKit.CRUD","doc":"* `new_comment/0` - Returns a new comment\n  * `new_comment/1` - Returns a new comment with params\n  * `new_comment/2` - Returns a new comment with params and opts","ref":"ContextKit.CRUD.html#module-new-operations"},{"type":"module","title":"List Operations - ContextKit.CRUD","doc":"* `list_comments/0` - Returns all comments\n  * `list_comments/1` - Returns filtered comments based on options","ref":"ContextKit.CRUD.html#module-list-operations"},{"type":"module","title":"Get Operations - ContextKit.CRUD","doc":"* `get_comment/1` - Fetches a single comment by ID\n  * `get_comment/2` - Fetches a comment by ID with additional filters\n  * `get_comment!/1` - Like `get_comment/1` but raises if not found\n  * `get_comment!/2` - Like `get_comment/2` but raises if not found","ref":"ContextKit.CRUD.html#module-get-operations"},{"type":"module","title":"Single Record Operations - ContextKit.CRUD","doc":"* `one_comment/1` - Fetches a single comment matching the criteria\n  * `one_comment!/1` - Like `one_comment/1` but raises if not found","ref":"ContextKit.CRUD.html#module-single-record-operations"},{"type":"module","title":"Save Operations - ContextKit.CRUD","doc":"* `save_comment/2` - Saves (inserts or updates) a comment with provided attributes\n  * `save_comment!/2` - Like `save_comment/2` but raises on invalid attributes","ref":"ContextKit.CRUD.html#module-save-operations"},{"type":"module","title":"Create Operations - ContextKit.CRUD","doc":"* `create_comment/1` - Creates a new comment with provided attributes\n  * `create_comment!/1` - Like `create_comment/1` but raises on invalid attributes","ref":"ContextKit.CRUD.html#module-create-operations"},{"type":"module","title":"Update Operations - ContextKit.CRUD","doc":"* `update_comment/2` - Updates comment with provided attributes\n  * `update_comment!/2` - Like `update_comment/2` but raises on invalid attributes","ref":"ContextKit.CRUD.html#module-update-operations"},{"type":"module","title":"Change Operations - ContextKit.CRUD","doc":"* `change_comment/2` - Returns a changeset for the comment with optional changes","ref":"ContextKit.CRUD.html#module-change-operations"},{"type":"module","title":"Delete Operations - ContextKit.CRUD","doc":"* `delete_comment/1` - Deletes a comment struct or by query criteria","ref":"ContextKit.CRUD.html#module-delete-operations"},{"type":"module","title":"Query Options - ContextKit.CRUD","doc":"All functions that accept options support:\n\n  * Basic filtering with field-value pairs\n  * Complex queries via `Ecto.Query`\n  * Pagination via `paginate: true` or `paginate: [page: 1, per_page: 20]`\n  * Custom query options defined in your queries module","ref":"ContextKit.CRUD.html#module-query-options"},{"type":"module","title":"Examples - ContextKit.CRUD","doc":"```elixir\n# Get a query for comments (for use with Repo.aggregate, etc.)\nquery = MyApp.Blog.query_comments(status: :published)\nMyApp.Repo.aggregate(query, :count)\n\n# New comment\nMyApp.Blog.new_comment()\n\n# List all comments\nMyApp.Blog.list_comments()\n\n# List published comments with pagination\nMyApp.Blog.list_comments(status: :published, paginate: [page: 1])\n\n# Get comment by ID with preloads\nMyApp.Blog.get_comment(123, preload: [:user])\n\n# Save a comment (insert if new, update if existing)\nMyApp.Blog.save_comment(comment, %{body: \"New or updated content\"})\n\n# Create a new comment\nMyApp.Blog.create_comment(%{body: \"Great post!\", user_id: 1})\n\n# Update a comment\nMyApp.Blog.update_comment(comment, %{body: \"Updated content\"})\n\n# Get a changeset for updates\nMyApp.Blog.change_comment(comment, %{body: \"Changed content\"})\n\n# Delete comment matching criteria\nMyApp.Blog.delete_comment(body: \"Specific content to delete\")\n```","ref":"ContextKit.CRUD.html#module-examples"},{"type":"module","title":"Queries Module - ContextKit.CRUD","doc":"The required `:queries` module should implement `apply_query_option/2`, which receives a query option and the current query and returns a modified query. This allows for custom filtering, sorting, and other query modifications.\n\n```elixir\ndefmodule MyApp.Blog.CommentQueries do\n  import Ecto.Query\n\n  def apply_query_option({:with_user_name, name}, query) do\n    query\n    |> join(:inner, [c], u in assoc(c, :user))\n    |> where([_, u], ilike(u.name, ^\"%#{name}%\"))\n  end\n\n  def apply_query_option({:recent_first, true}, query) do\n    order_by(query, [c], desc: c.inserted_at)\n  end\n\n  def apply_query_option(_, query), do: query\nend\n```\n\nEach generated function can be overridden in your context module if you need custom behavior.","ref":"ContextKit.CRUD.html#module-queries-module"},{"type":"module","title":"ContextKit.CRUD.Scoped","doc":"The `ContextKit.CRUD.Scoped` module provides a convenient way to generate standard CRUD (Create, Read, Update, Delete) operations for your Ecto schemas. It reduces boilerplate code by automatically generating commonly used database interaction functions.\n\nAdditionally, it supports scopes from Phoenix 1.8.","ref":"ContextKit.CRUD.Scoped.html"},{"type":"module","title":"Setup - ContextKit.CRUD.Scoped","doc":"Add the following to your context module:\n\n```elixir\ndefmodule MyApp.Blog do\n  use ContextKit.CRUD.Scoped,\n    repo: MyApp.Repo,\n    schema: MyApp.Blog.Comment,\n    queries: MyApp.Blog.CommentQueries,\n    pubsub: MyApp.PubSub,                 # For realtime notifications via PubSub\n    scope: Application.compile_env(:my_app, :scopes)[:user], # To gain support for Phoenix 1.8 scopes.\n    except: [:delete],                    # Optional: exclude specific operations\n    plural_resource_name: \"comments\"      # Optional: customize plural name\nend\n```","ref":"ContextKit.CRUD.Scoped.html#module-setup"},{"type":"module","title":"Required Options - ContextKit.CRUD.Scoped","doc":"* `:repo` - The Ecto repository module to use for database operations\n  * `:schema` - The Ecto schema module that defines your resource\n  * `:queries` - Module containing query-building functions for advanced filtering\n  * `:pubsub` - The Phoenix.PubSub module to use for real-time features (required for subscription features)\n  * `:scope` - Configuration for scoping resources to specific contexts (e.g., user)","ref":"ContextKit.CRUD.Scoped.html#module-required-options"},{"type":"module","title":"Optional Options - ContextKit.CRUD.Scoped","doc":"* `:except` - List of operation types to exclude (`:new`, `:list`, `:get`, `:one`, `:delete`, `:create`, `:update`, `:change`, `:subscribe`, `:broadcast`)\n  * `:plural_resource_name` - Custom plural name for list functions (defaults to singular + \"s\")","ref":"ContextKit.CRUD.Scoped.html#module-optional-options"},{"type":"module","title":"Generated Functions - ContextKit.CRUD.Scoped","doc":"For a schema named `Comment`, the following functions are generated:","ref":"ContextKit.CRUD.Scoped.html#module-generated-functions"},{"type":"module","title":"Query Operations - ContextKit.CRUD.Scoped","doc":"* `query_comments/0` - Returns a base query for all comments\n  * `query_comments/1` - Returns a filtered query based on options (without executing)\n  * `query_comments/2` - Returns a scoped and filtered query if `:scope` is configured","ref":"ContextKit.CRUD.Scoped.html#module-query-operations"},{"type":"module","title":"New Operations - ContextKit.CRUD.Scoped","doc":"* `new_comment/0` - Returns a new comment\n  * `new_comment/1` - Returns a new comment with params\n  * `new_comment/2` - Returns a new comment with params and opts","ref":"ContextKit.CRUD.Scoped.html#module-new-operations"},{"type":"module","title":"List Operations - ContextKit.CRUD.Scoped","doc":"* `list_comments/0` - Returns all comments\n  * `list_comments/1` - Returns filtered comments based on options\n  * `list_comments/2` - Returns scoped and filtered comments if `:scope` is configured","ref":"ContextKit.CRUD.Scoped.html#module-list-operations"},{"type":"module","title":"Get Operations - ContextKit.CRUD.Scoped","doc":"* `get_comment/1` - Fetches a single comment by ID\n  * `get_comment/2` - Fetches a comment by ID with additional filters\n  * `get_comment/3` - Fetches a scoped comment by ID with additional filters if `:scope` is configured\n  * `get_comment!/1` - Like `get_comment/1` but raises if not found\n  * `get_comment!/2` - Like `get_comment/2` but raises if not found\n  * `get_comment!/3` - Like `get_comment/3` but raises if not found (with scope)","ref":"ContextKit.CRUD.Scoped.html#module-get-operations"},{"type":"module","title":"Single Record Operations - ContextKit.CRUD.Scoped","doc":"* `one_comment/1` - Fetches a single comment matching the criteria\n  * `one_comment/2` - Fetches a scoped single comment if `:scope` is configured\n  * `one_comment!/1` - Like `one_comment/1` but raises if not found\n  * `one_comment!/2` - Like `one_comment/2` but raises if not found (with scope)","ref":"ContextKit.CRUD.Scoped.html#module-single-record-operations"},{"type":"module","title":"Save Operations - ContextKit.CRUD.Scoped","doc":"* `save_comment/1` - Saves (inserts or updates) a comment\n  * `save_comment/2` - Saves a comment with the provided attributes\n  * `save_comment/3` - Saves a scoped comment with attributes if `:scope` is configured\n  * `save_comment!/1` - Like `save_comment/1` but raises on invalid attributes\n  * `save_comment!/2` - Like `save_comment/2` but raises on invalid attributes\n  * `save_comment!/3` - Like `save_comment/3` but raises on invalid attributes (with scope)","ref":"ContextKit.CRUD.Scoped.html#module-save-operations"},{"type":"module","title":"Create Operations - ContextKit.CRUD.Scoped","doc":"* `create_comment/1` - Creates a new comment with provided attributes\n  * `create_comment/2` - Creates a scoped comment if `:scope` is configured\n  * `create_comment!/1` - Like `create_comment/1` but raises on invalid attributes\n  * `create_comment!/2` - Like `create_comment/2` but raises on invalid attributes (with scope)","ref":"ContextKit.CRUD.Scoped.html#module-create-operations"},{"type":"module","title":"Update Operations - ContextKit.CRUD.Scoped","doc":"* `update_comment/2` - Updates comment with provided attributes\n  * `update_comment/3` - Updates scoped comment if `:scope` is configured\n  * `update_comment!/2` - Like `update_comment/2` but raises on invalid attributes\n  * `update_comment!/3` - Like `update_comment/3` but raises on invalid attributes (with scope)","ref":"ContextKit.CRUD.Scoped.html#module-update-operations"},{"type":"module","title":"Change Operations - ContextKit.CRUD.Scoped","doc":"* `change_comment/1` - Returns a changeset for the comment\n  * `change_comment/2` - Returns a changeset for the comment with changes\n  * `change_comment/3` - Returns a changeset for the scoped comment with changes if `:scope` is configured","ref":"ContextKit.CRUD.Scoped.html#module-change-operations"},{"type":"module","title":"Delete Operations - ContextKit.CRUD.Scoped","doc":"* `delete_comment/1` - Deletes a comment struct or by query criteria\n  * `delete_comment/2` - Deletes a scoped comment if `:scope` is configured","ref":"ContextKit.CRUD.Scoped.html#module-delete-operations"},{"type":"module","title":"PubSub Operations (if `:pubsub` and `:scope` are configured) - ContextKit.CRUD.Scoped","doc":"* `subscribe_comments/1` - Subscribes to the scoped comments topic\n  * `broadcast_comment/2` - Broadcasts a message to the scoped comments topic","ref":"ContextKit.CRUD.Scoped.html#module-pubsub-operations-if-pubsub-and-scope-are-configured"},{"type":"module","title":"Query Options - ContextKit.CRUD.Scoped","doc":"All functions that accept options support:\n\n  * Basic filtering with field-value pairs\n  * Complex queries via `Ecto.Query`\n  * Pagination via `paginate: true` or `paginate: [page: 1, per_page: 20]`\n  * Custom query options defined in your queries module\n  * Scoping via `scope` when using scoped functions","ref":"ContextKit.CRUD.Scoped.html#module-query-options"},{"type":"module","title":"Examples - ContextKit.CRUD.Scoped","doc":"```elixir\n# Get a query for comments (for use with Repo.aggregate, etc.)\nquery = MyApp.Blog.query_comments(status: :published)\nMyApp.Repo.aggregate(query, :count)\n\n# Get a query for comments scoped to current user\nquery = MyApp.Blog.query_comments(socket.assigns.current_scope)\nMyApp.Repo.aggregate(query, :count)\n\n# Get a query for comments scoped to current user with additional filters\nquery = MyApp.Blog.query_comments(socket.assigns.current_scope, status: :published)\nMyApp.Repo.aggregate(query, :count)\n\n# New comment\nMyApp.Blog.new_comment()\n\n# List all comments\nMyApp.Blog.list_comments()\n\n# List all comments belonging to current user\nMyApp.Blog.list_comments(socket.assigns.current_scope)\n\n# List published comments with pagination\nMyApp.Blog.list_comments(status: :published, paginate: [page: 1])\n\n# Get comment by ID with preloads\nMyApp.Blog.get_comment(123, preload: [:user])\n\n# Get comment by ID that belongs to current user\nMyApp.Blog.get_comment(socket.assigns.current_scope, 123)\n\n# Save a new comment (will be inserted)\nMyApp.Blog.save_comment(%Comment{}, %{body: \"Great post!\", user_id: 42})\n\n# Save an existing comment (will be updated)\nMyApp.Blog.save_comment(existing_comment, %{body: \"Updated content\"})\n\n# Save a comment with scope (will insert or update depending on the record's state)\nMyApp.Blog.save_comment(socket.assigns.current_scope, comment, %{body: \"Content\"})\n\n# Save a comment or raise on errors\nMyApp.Blog.save_comment!(comment, %{body: \"Content that must be saved\"})\n\n# Create a new comment (manually specifying user_id)\nMyApp.Blog.create_comment(%{body: \"Great post!\", user_id: 42})\n\n# Create a new comment that automatically belongs to current user\nMyApp.Blog.create_comment(socket.assigns.current_scope, %{body: \"Great post!\"})\n\n# Update a comment\nMyApp.Blog.update_comment(comment, %{body: \"Updated content\"})\n\n# Update a comment that belongs to current user\nMyApp.Blog.update_comment(socket.assigns.current_scope, comment, %{body: \"Updated content\"})\n\n# Get a changeset for updates\nMyApp.Blog.change_comment(comment, %{body: \"Changed content\"})\n\n# Get a changeset for updates with scope\nMyApp.Blog.change_comment(socket.assigns.current_scope, comment, %{body: \"Changed content\"})\n\n# Delete comment\nMyApp.Blog.delete_comment(comment)\n\n# Delete comment that belongs to current user\nMyApp.Blog.delete_comment(socket.assigns.current_scope, comment)\n\n# Delete comment matching criteria\nMyApp.Blog.delete_comment(body: \"Specific content to delete\")\n```\n\nEach generated function can be overridden in your context module if you need custom behavior.","ref":"ContextKit.CRUD.Scoped.html#module-examples"},{"type":"module","title":"Queries Module - ContextKit.CRUD.Scoped","doc":"The required `:queries` module should implement `apply_query_option/2`, which receives a query option and the current query and returns a modified query. This allows for custom filtering, sorting, and other query modifications.\n\n```elixir\ndefmodule MyApp.Blog.CommentQueries do\n  import Ecto.Query\n\n  def apply_query_option({:with_user_name, name}, query) do\n    query\n    |> join(:inner, [c], u in assoc(c, :user))\n    |> where([_, u], ilike(u.name, ^\"%#{name}%\"))\n  end\n\n  def apply_query_option({:recent_first, true}, query) do\n    order_by(query, [c], desc: c.inserted_at)\n  end\n\n  def apply_query_option(_, query), do: query\nend\n```","ref":"ContextKit.CRUD.Scoped.html#module-queries-module"},{"type":"module","title":"Scope - ContextKit.CRUD.Scoped","doc":"[Read more about scopes](https://hexdocs.pm/phoenix/1.8.0-rc.0/scopes.html).\n\nA scope is a data structure used to keep information about the current request or session, such as the current user logged in, permissions, and so on. By using scopes, you have a single data structure that contains all relevant information, which is then passed around so all of your operations are properly scoped.\n\nUsually you configure it with the same scope that you use in your Phoenix application:\n\n```elixir\nscope: Application.compile_env(:my_app, :scopes)[:user],\npubsub: MyApp.PubSub # pubsub config is required for scopes\n```\n\nWhen a scope is configured, all relevant CRUD functions take an additional scope parameter as their first argument, ensuring that operations only affect records that belong to that scope.\n\nFor our comments example, the scope makes sure that comments are only listed, edited, or deleted by the user who created them.","ref":"ContextKit.CRUD.Scoped.html#module-scope"},{"type":"module","title":"Scope Configuration - ContextKit.CRUD.Scoped","doc":"If you pass in the scope with `Application.compile_env(:my_app, :scopes)[:user]` - it will work automatically. You can also configure the scope manually. The scope configuration should be a keyword list with the following keys:\n\n* `:module` - The module that defines the scope struct\n* `:access_path` - Path to access the scoping value (e.g., `[:user, :id]` for user ID)\n* `:schema_key` - The field in the schema that corresponds to the scope (e.g., `:user_id`)\n\nFor our comments example, this would typically be:\n\n```elixir\nscope: [\n  module: MyApp.Accounts.Scope,\n  access_path: [:user, :id],\n  schema_key: :user_id\n]\n```","ref":"ContextKit.CRUD.Scoped.html#module-scope-configuration"},{"type":"module","title":"Subscription Example - ContextKit.CRUD.Scoped","doc":"```elixir\n# Subscribe to comments created, updated, or deleted by the current user\nMyApp.Blog.subscribe_comments(socket.assigns.current_scope)\n\n# Now the current process will receive messages like:\n# {:created, %Comment{}}\n# {:updated, %Comment{}}\n# {:deleted, %Comment{}}\n```","ref":"ContextKit.CRUD.Scoped.html#module-subscription-example"},{"type":"module","title":"Broadcasting Example - ContextKit.CRUD.Scoped","doc":"```elixir\n# Broadcast a custom message to all subscribers for current user's comments\nMyApp.Blog.broadcast_comment(socket.assigns.current_scope, {:custom_event, comment})\n```\n\nFollowing messages are also broadcasted automatically for all create, update, delete operations:\n\n```elixir\n{:created, %Comment{}}\n{:updated, %Comment{}}\n{:deleted, %Comment{}}\n```","ref":"ContextKit.CRUD.Scoped.html#module-broadcasting-example"},{"type":"module","title":"ContextKit.Paginator","doc":"Handles pagination for database queries in ContextKit.\n\nThis module provides pagination functionality with configurable page size,\nautomatic limit/offset calculation, and metadata about the paginated results.","ref":"ContextKit.Paginator.html"},{"type":"function","title":"ContextKit.Paginator.changed?/2","doc":"","ref":"ContextKit.Paginator.html#changed?/2"},{"type":"function","title":"ContextKit.Paginator.new/3","doc":"","ref":"ContextKit.Paginator.html#new/3"},{"type":"function","title":"ContextKit.Paginator.params/1","doc":"","ref":"ContextKit.Paginator.html#params/1"},{"type":"type","title":"ContextKit.Paginator.t/0","doc":"","ref":"ContextKit.Paginator.html#t:t/0"},{"type":"module","title":"ContextKit.Query","doc":"Provides dynamic query building functionality with extensive filtering operations for Ecto queries.\n\nThis module is core to ContextKit's filtering capabilities, providing a flexible way to build\ncomplex database queries from runtime parameters.","ref":"ContextKit.Query.html"},{"type":"module","title":"Query Building - ContextKit.Query","doc":"The query building process consists of:\n1. Creating a base query with proper binding names\n2. Applying field filters based on schema fields\n3. Applying standard query options (order_by, limit, etc.)\n4. Applying custom query options","ref":"ContextKit.Query.html#module-query-building"},{"type":"module","title":"Field Filters - ContextKit.Query","doc":"Supports an extensive set of filter operations through a standardized filter syntax:\n\n```elixir\n%{field: field_name, op: operator, value: value}\n```","ref":"ContextKit.Query.html#module-field-filters"},{"type":"module","title":"Available Operators - ContextKit.Query","doc":"Basic Comparison:\n- `:==` - Equality\n- `:!=` - Inequality\n- `:<` - Less than\n- `:<=` - Less than or equal\n- `:>` - Greater than\n- `:>=` - Greater than or equal\n\nNull Checks:\n- `:empty` - Check if field is NULL\n- `:not_empty` - Check if field is NOT NULL\n\nList Operations:\n- `:in` - Check if field value is in list\n- `:not_in` - Check if field value is not in list\n- `:contains` - Check if list field contains value\n- `:not_contains` - Check if list field does not contain value\n\nPattern Matching:\n- `:like` - Case-sensitive pattern matching\n- `:not_like` - Negative case-sensitive pattern matching\n- `:ilike` - Case-insensitive pattern matching\n- `:not_ilike` - Negative case-insensitive pattern matching\n- `:=~` - Shorthand for `:ilike`\n\nMultiple Pattern Matching:\n- `:like_and` - All patterns must match (case-sensitive)\n- `:like_or` - Any pattern must match (case-sensitive)\n- `:ilike_and` - All patterns must match (case-insensitive)\n- `:ilike_or` - Any pattern must match (case-insensitive)","ref":"ContextKit.Query.html#module-available-operators"},{"type":"module","title":"Standard Query Options - ContextKit.Query","doc":"The following standard query options are supported:\n\n  - `:order_by` - Specify sort order\n  - `:group_by` - Group results\n  - `:limit` - Limit number of results\n  - `:preload` - Preload associations\n  - `:paginate` - Enable pagination with optional configuration","ref":"ContextKit.Query.html#module-standard-query-options"},{"type":"function","title":"ContextKit.Query.build/3","doc":"","ref":"ContextKit.Query.html#build/3"},{"type":"function","title":"ContextKit.Query.field_filter_keys/1","doc":"","ref":"ContextKit.Query.html#field_filter_keys/1"},{"type":"function","title":"ContextKit.Query.new/1","doc":"","ref":"ContextKit.Query.html#new/1"},{"type":"function","title":"ContextKit.Query.query_options_keys/1","doc":"","ref":"ContextKit.Query.html#query_options_keys/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}